/*
 * Copyright (C) 2013  Waritnan Sookbuntherng
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package tk.lion328.game.engine.coalsrc;

import java.util.Map.Entry;
import java.util.logging.Logger;

import org.lwjgl.LWJGLException;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.lwjgl.util.glu.GLU;
import org.lwjgl.util.vector.Vector3f;

import tk.lion328.game.engine.coalsrc.render3d.Camera;
import tk.lion328.game.engine.coalsrc.render3d.GameObject;
import tk.lion328.game.engine.coalsrc.ui.UIObject;

public class GameRuntime {
	
	private Logger logger;
	private static GameRuntime $;
	
	public GameRuntime(GameApplication app, String gameName, String title, int width, int height, Camera cam) throws LWJGLException{
		$ = this;
		
		logger = Logger.getLogger(gameName);
		
		logger.info("CoalSrc " + Constant.ENGINE_VERSION);
		logger.info("Started by \"" + gameName + "\"");
		logger.info("Creating window...");
		
		Display.setDisplayMode(new DisplayMode(width,height));
		Display.setTitle(title);
		Display.create();
		
		logger.info("Start game runtime.");
		
		while(!Display.isCloseRequested()){
			GL11.glMatrixMode(GL11.GL_PROJECTION);
			GL11.glLoadIdentity();

			GLU.gluPerspective((float) 100, width / height, 0.001f, 1000);
			GL11.glMatrixMode(GL11.GL_MODELVIEW);

			GL11.glEnable(GL11.GL_TEXTURE_2D);
			GL11.glShadeModel(GL11.GL_SMOOTH);
			GL11.glClearColor(0.0f, 0.0f, 0.0f, 0.5f);
			GL11.glClearDepth(1.0f);
			GL11.glEnable(GL11.GL_DEPTH_TEST);
			GL11.glDepthFunc(GL11.GL_LEQUAL);
			
			GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);
			GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_POLYGON);
			GL11.glLoadIdentity();
			
			cam.translateCamera();
			
			for(Entry<Vector3f,GameObject> entry : app.getGameObjectManager().getObjectsList().entrySet()){
				entry.getValue().update();
				entry.getValue().render();
			}
			
			GL11.glPushMatrix();
			
			GL11.glMatrixMode(GL11.GL_PROJECTION);
			GL11.glLoadIdentity();
			GL11.glOrtho(0, width, 0, height, -1, 1);
			GL11.glMatrixMode(GL11.GL_MODELVIEW);
			GL11.glEnable(GL11.GL_TEXTURE_2D);
			GL11.glLoadIdentity();
			
			for(Entry<String,UIObject> entry : app.getIngameUI().getOverlayObjectsList().entrySet()){
				entry.getValue().update();
				entry.getValue().render();
			}
			
			Display.update();
			Display.sync(90);
		}
	}
	
	public Logger getLogger(){
		return logger;
	}
	
	public static GameRuntime getCurrentRuntime(){
		return $;
	}
	
}
