/*
 * Copyright (C) 2013  Waritnan Sookbuntherng
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package tk.lion328.game.engine.coalsrc.render3d;

import java.util.Map.Entry;

import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.GL11;
import org.lwjgl.util.vector.Vector2f;
import org.lwjgl.util.vector.Vector3f;

import tk.lion328.game.engine.coalsrc.controller.KeyboardController;
import tk.lion328.game.engine.coalsrc.setting.Setting;

public class Camera {
	
	private float speed = 0.06F;
	
	private boolean moveForward = false, moveBackward = false, strafeLeft = false, strafeRight = false;
	private boolean onGround = false, jumping = false, falling = false;
	
	private float yBottom = 0, yMotion = 0, gravity = 0.0138F;
	
	private Vector2f mapSize;
	
	private Vector3f vector = new Vector3f(7,1,7), rotation = new Vector3f(), previous = new Vector3f();
	
	private GameObjectManager objman;
	private KeyboardController keyboard;
	private Setting setting;
	
	public Camera(float gravity, Setting setting, KeyboardController keyboard, GameObjectManager objs, Vector2f mapSize, Vector3f spawnpoint){
		this.mapSize = mapSize;
		this.gravity = gravity;
		vector = spawnpoint;
		this.setting = setting;
	}
	
	public void setCurrentSpeed(float s){
		speed = s;
	}
	
	public float getCurrentSpeed(){
		return speed;
	}
	
	public void update(){
		updatePreviousVector();
		updateMotion();
		input();
		collisions(-0.9f);
		individualPhysics();
	}
	
	public void input(){
		moveForward = keyboard.isKeyDown(Integer.parseInt(setting.getSettingValue("control.forward")));
		moveBackward = keyboard.isKeyDown(Integer.parseInt(setting.getSettingValue("control.backward")));
		strafeLeft = keyboard.isKeyDown(Integer.parseInt(setting.getSettingValue("control.left")));
		strafeRight = keyboard.isKeyDown(Integer.parseInt(setting.getSettingValue("control.right")));
		yMotion = keyboard.isKeyDown(Integer.parseInt(setting.getSettingValue("control.jump"))) && onGround ? 0.26F : yMotion;

		if(Mouse.isGrabbed()){
			float mouseDX = Mouse.getDX() * 0.8f * 0.16f;
			float mouseDY = Mouse.getDY() * 0.8f * 0.16f;
			if (rotation.y + mouseDX >= 360) {
				rotation.y = rotation.y + mouseDX - 360;
			} else if (rotation.y + mouseDX < 0) {
				rotation.y = 360 - rotation.y + mouseDX;
			} else {
				rotation.y += mouseDX;
			}
			if (rotation.x - mouseDY >= -89 && rotation.x - mouseDY <= 89) {
				rotation.x += -mouseDY;
			} else if (rotation.x - mouseDY < -89) {
				rotation.x = -89;
			} else if (rotation.x - mouseDY > 89) {
				rotation.x = 89;
			}
		}
	}
	
	private boolean ontopOfCollidablePoly(float offset) {
		for(Entry<Vector3f, GameObject> entry : objman.getObjectsList().entrySet()){
			GameObject v = entry.getValue();
			if(Math.abs(vector.x - v.getCenterPos().getY()) < v.getWidth()/2 && Math.abs(vector.z - v.getCenterPos().getZ()) < v.getLength()/2){
				return true;
			}
		}
		return false;
	}
	
	public void collisions(float offset){
		if(!ontopOfCollidablePoly(offset)){
			yBottom = 0;
		}
		for(Entry<Vector3f, GameObject> entry : objman.getObjectsList().entrySet()){
			GameObject v = entry.getValue();
			if(Math.abs(vector.x - v.getCenterPos().getX()) < v.getWidth()/2 && Math.abs(vector.z - v.getCenterPos().getZ()) < v.getLength()/2){
				if( ( Math.abs(vector.y - v.getyTop()*2) < v.getHeight()+(offset)) || vector.y > v.getyBottom()+v.getHeight()-0.5f){
					yBottom = v.getStartPos().getY()+v.getyTop();
				}else if(v.getyBottom() > vector.y+2.4f){
					
				}else{
					vector.x = previous.x;
					vector.z = previous.z;
				}
			}
		}
	}

	public void updatePreviousVector(){
		previous = new Vector3f(vector);
	}
	
	public void translateCamera(){
		GL11.glRotatef(rotation.x, 1, 0, 0);
		GL11.glRotatef(rotation.y, 0, 1, 0);
		GL11.glRotatef(rotation.z, 0, 0, 1);
		GL11.glTranslatef(-vector.x, -vector.y - 1.4f, -vector.z);
	}
	
	public void individualPhysics(){
		onGround = (vector.y == yBottom && !jumping && !falling);
		jumping = (yMotion > 0);
		falling = (yMotion < 0);
		vector.y += yMotion;
		if(vector.y > yBottom){
			yMotion -= gravity;
		}
		if(vector.y <= yBottom){
			vector.y = yBottom;
			yMotion = 0;
		}
	}
	
	public void updateMotion(){
		if(moveForward){
			vector.x += Math.sin(rotation.y * Math.PI/180) * speed;
			vector.z += -Math.cos(rotation.y * Math.PI/180) * speed;
		}
		if(moveBackward){
			vector.x -= Math.sin(rotation.y * Math.PI/180) * speed;
			vector.z -= -Math.cos(rotation.y * Math.PI/180) * speed;
		}
		if(strafeLeft){
			vector.x += Math.sin((rotation.y-90) * Math.PI/180) * speed;
			vector.z += -Math.cos((rotation.y-90) * Math.PI/180) * speed;
		}
		if(strafeRight){
			vector.x += Math.sin((rotation.y+90) * Math.PI/180) * speed;
			vector.z += -Math.cos((rotation.y+90) * Math.PI/180) * speed;
		}
		if(vector.x > mapSize.x || vector.x < 0 || vector.z > mapSize.y || vector.z < 0 ){
			vector.x = previous.x;
			vector.z = previous.z;
		}
	}
	
	public void setGravity(float g){
		gravity = g;
	}
	
	public float getGravity(){
		return gravity;
	}
}
